/*
 * Sentinella
 * http://sentinella.sourceforge.net/
 * Copyright (c) 2009-2011 Carlos Olmedo Escobar <carlos.olmedo.e@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foun dation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "MainWindow.h"
#include <errno.h>
#include <QFile>
#include <QDBusInterface>
#include <QDBusReply>
#include <QNetworkInterface>
#include <QSetIterator>
#include <QTreeView>
#include <kmenu.h>
#include <kaction.h>
#include <KUrl>
#include <kfiledialog.h>
#include <kstandardaction.h>
#include <kactioncollection.h>
#include <KConfig>
#include <KSharedConfig>
#include <kpassivepopup.h>
#include <kdialog.h>
#include <kaboutapplicationdialog.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <ksysguard/ksysguardprocesslist.h>
#include <kiconloader.h>
#include "Conditions/CPU.h"
#include "Conditions/DateTime.h"
#include "Conditions/Memory.h"
#include "Conditions/Network.h"
#include "Conditions/ProcessDies.h"
#include "Actions/ExecuteCommand.h"
#include "Actions/KillProcess.h"
#include "Actions/PlayAlarm.h"
#include "Actions/Shutdown.h"
#include "Actions/Sleep.h"
#include <iostream>

// Ids of the condition radiobuttons
#define CONDITION_CPU_ID 0
#define CONDITION_MEMORY_ID 1
#define CONDITION_NETWORK_ID 2
#define CONDITION_DATETIME_ID 3
#define CONDITION_PROCESSDIES_ID 4

// Ids of the action radiobuttons
#define ACTION_SHUTDOWN_ID 0
#define ACTION_REBOOT_ID 1
#define ACTION_SLEEP_ID 2
#define ACTION_EXECUTECOMMAND_ID 3
#define ACTION_PLAYALARM_ID 4
#define ACTION_KILLPROCESS_ID 5

MainWindow::MainWindow(const KAboutData& aboutData, KCmdLineArgs* cmdLineArgs,
		const short cmdChosenCondition, const short cmdChosenAction) :
	KMainWindow(NULL, Qt::WindowContextHelpButtonHint | Qt::WindowCloseButtonHint), aboutData(
			aboutData), cmdLineArgs(cmdLineArgs) {
	condition = NULL;
	action = NULL;
	supportedShutdown = false;
	pidProcessDies = pidKillProcess = 0;
	availableSounds = NULL;
	processAPI = NULL;
	networkInterfaceAPI = NULL;

	checkShutdownSleepStates();
	iconLoader = KIconLoader::global();

	config = KGlobal::config();
	KConfigGroup windowGroup = config->group("Window");
	if (windowGroup.readEntry("Visible", true))
		show();

	gui.setupUi(this);
	gui.pushButtonStartStop->setIcon(QIcon(iconLoader->loadIcon("arrow-right", KIconLoader::Small)));

	trayIcon.setAssociatedWidget(this);
	trayIcon.setCategory(KStatusNotifierItem::ApplicationStatus);
	trayIcon.setStatus(KStatusNotifierItem::Active);
	trayIcon.contextMenu()->addAction(KStandardAction::aboutApp(this, SLOT(showAboutApp()),
			trayIcon.contextMenu()));
	trayIcon.setIconByName("sentinella");

	conditionsGroup.addButton(gui.radioButtonCPU, CONDITION_CPU_ID);
	conditionsGroup.addButton(gui.radioButtonMemory, CONDITION_MEMORY_ID);
	conditionsGroup.addButton(gui.radioButtonNetwork, CONDITION_NETWORK_ID);
	conditionsGroup.addButton(gui.radioButtonDateTime, CONDITION_DATETIME_ID);
	conditionsGroup.addButton(gui.radioButtonProcessDies, CONDITION_PROCESSDIES_ID);
	connect(gui.radioButtonNetwork, SIGNAL(toggled(bool)), this, SLOT(radioButtonNetworkToggled(bool)));
	connect(&conditionsGroup, SIGNAL(buttonClicked(int)), gui.stackedWidgetConditions, SLOT(setCurrentIndex(int)));
	connect(gui.pushButtonProcessDies, SIGNAL(clicked(bool)), this, SLOT(programDiesClicked()));

	if (supportedShutdown) {
		actionsGroup.addButton(gui.radioButtonShutdown, ACTION_SHUTDOWN_ID);
		actionsGroup.addButton(gui.radioButtonReboot, ACTION_REBOOT_ID);
	} else {
		gui.radioButtonShutdown->setEnabled(false);
		gui.radioButtonReboot->setEnabled(false);
	}
	if (supportedSleepStates.empty())
		gui.radioButtonSleep->setEnabled(false);
	else
		actionsGroup.addButton(gui.radioButtonSleep, ACTION_SLEEP_ID);
	actionsGroup.addButton(gui.radioButtonExecuteCommand, ACTION_EXECUTECOMMAND_ID);
	actionsGroup.addButton(gui.radioButtonPlayAlarm, ACTION_PLAYALARM_ID);
	actionsGroup.addButton(gui.radioButtonKillProcess, ACTION_KILLPROCESS_ID);
	connect(gui.radioButtonPlayAlarm, SIGNAL(toggled(bool)), this, SLOT(radioButtonPlayAlarmToggled(bool)));
	connect(&actionsGroup, SIGNAL(buttonClicked(int)), this, SLOT(actionButtonClicked(int)));
	connect(gui.pushButtonKillProcess, SIGNAL(clicked(bool)), this, SLOT(killProcessClicked()));

	connect(gui.pushButtonStartStop, SIGNAL(clicked(bool)), this, SLOT(startStopClicked()));

	initConditionWidgets(cmdChosenCondition);
	initActionWidgets(cmdChosenAction);
	if (cmdLineArgs->isSet("start"))
		startClicked();

	cmdLineArgs->clear(); // the given cmd args are not needed anymore
}

MainWindow::~MainWindow() {
	KConfigGroup windowGroup = config->group("Window");
	bool configIsWindowVisible = windowGroup.readEntry("Visible", true);
	if (configIsWindowVisible != isVisible())
		windowGroup.writeEntry("Visible", !configIsWindowVisible);

	delete availableSounds;
	delete processAPI;
	delete networkInterfaceAPI;
}

void MainWindow::checkShutdownSleepStates() {
	//see http://majewsky.wordpress.com/2009/07/11/shutdown-your-machine-automatically-or-from-remote/
	QDBusInterface* interface = new QDBusInterface("org.kde.ksmserver", "/KSMServer",
			"org.kde.KSMServerInterface");
	QDBusReply<bool> boolReply = interface->call("canShutdown");
	if (boolReply.isValid()) {
		supportedShutdown = boolReply;
		isKDE = true;
		delete interface;

		interface = new QDBusInterface("org.kde.Solid.PowerManagement",
				"/org/freedesktop/PowerManagement", "org.freedesktop.PowerManagement");
		boolReply = interface->call("CanSuspend");
		if (boolReply.isValid() && boolReply)
			supportedSleepStates.insert(Solid::PowerManagement::SuspendState);
		boolReply = interface->call("CanHibernate");
		if (boolReply.isValid() && boolReply)
			supportedSleepStates.insert(Solid::PowerManagement::HibernateState);
	} else {
		//see http://upower.freedesktop.org/docs/UPower.html
		delete interface;
		interface = new QDBusInterface("org.freedesktop.ConsoleKit",
				"/org/freedesktop/ConsoleKit/Manager", "org.freedesktop.ConsoleKit.Manager",
				QDBusConnection::systemBus());
		boolReply = interface->call("CanStop");
		if (boolReply.isValid()) {
			supportedShutdown = boolReply;
			isKDE = false;
			delete interface;

			interface = new QDBusInterface("org.freedesktop.UPower", "/org/freedesktop/UPower",
					"org.freedesktop.UPower", QDBusConnection::systemBus());
			boolReply = interface->call("SuspendAllowed");
			if (boolReply.isValid() && boolReply)
				supportedSleepStates.insert(Solid::PowerManagement::SuspendState);
			boolReply = interface->call("HibernateAllowed");
			if (boolReply.isValid() && boolReply)
				supportedSleepStates.insert(Solid::PowerManagement::HibernateState);
		}
	}

	delete interface;
}

void MainWindow::initConditionWidgets(const short cmdChosenCondition) {
	int aux;
	if (cmdChosenCondition != -1)
		aux = cmdChosenCondition;
	else {
		KConfigGroup group = config->group("Conditions");
		aux = group.readEntry("LastExecuted", 0);
		if (aux < CONDITION_CPU_ID || aux > CONDITION_PROCESSDIES_ID)
			aux = CONDITION_CPU_ID;
	}

	conditionsGroup.button(aux)->setChecked(true);
	gui.stackedWidgetConditions->setCurrentIndex(aux);

	if (cmdLineArgs->isSet("higher"))
		aux = 0;
	else if (cmdLineArgs->isSet("lower"))
		aux = 1;
	else
		aux = -1;

	KConfigGroup groupCPU = config->group("ConditionCPU");
	gui.comboBoxCPU->setCurrentIndex(aux != -1 ? aux : groupCPU.readEntry("Higher", 1));
	gui.spinBoxCPU->setValue(
			cmdChosenCondition == CONDITION_CPU_ID ? cmdLineArgs->getOption("cpu").toInt()
					: groupCPU.readEntry("Percentage", 10));
	gui.timeEditCPU->setTime(QTime::fromString(cmdChosenCondition == CONDITION_CPU_ID
			&& cmdLineArgs->isSet("during") ? cmdLineArgs->getOption("during")
			: groupCPU.readEntry("Time", "01:00"), "mm:ss"));

	KConfigGroup groupMemory = config->group("ConditionMemory");
	gui.comboBoxMemory->setCurrentIndex(aux != -1 ? aux : groupMemory.readEntry("Higher", 1));
	gui.spinBoxMemory->setValue(cmdChosenCondition == CONDITION_MEMORY_ID ? cmdLineArgs->getOption(
			"memory").toInt() : groupMemory.readEntry("Amount", 512));
	gui.timeEditMemory->setTime(QTime::fromString(cmdChosenCondition == CONDITION_MEMORY_ID
			&& cmdLineArgs->isSet("during") ? cmdLineArgs->getOption("during")
			: groupMemory.readEntry("Time", "01:00"), "mm:ss"));

	KConfigGroup groupNetwork = config->group("ConditionNetwork");
	gui.comboBoxNetworkDirection->setCurrentIndex(groupNetwork.readEntry("Direction", 0));
	gui.comboBoxNetworkHigher->setCurrentIndex(aux != -1 ? aux
			: groupNetwork.readEntry("Higher", 0));
	gui.spinBoxNetwork->setValue(
			cmdChosenCondition == CONDITION_NETWORK_ID ? cmdLineArgs->getOption("network").toInt()
					: groupNetwork.readEntry("Amount", 10));
	gui.timeEditNetwork->setTime(QTime::fromString(cmdChosenCondition == CONDITION_NETWORK_ID
			&& cmdLineArgs->isSet("during") ? cmdLineArgs->getOption("during")
			: groupNetwork.readEntry("Time", "01:00"), "mm:ss"));

	QDateTime dateTime;
	if (cmdChosenCondition == CONDITION_DATETIME_ID)
		dateTime = QDateTime::fromString(cmdLineArgs->getOption("date-time"), Qt::ISODate);
	if (cmdChosenCondition != CONDITION_DATETIME_ID || !dateTime.isValid()) {
		KConfigGroup groupDateTime = config->group("ConditionDateTime");
		dateTime = QDateTime::currentDateTime().addSecs(
				groupDateTime.readEntry("Interval", "1800").toInt());
	}
	QTime readTime(dateTime.time().hour(), dateTime.time().minute(), 0); // Setting 0 seconds
	dateTime.setTime(readTime);
	gui.kdatetimewidgetDateTime->setDateTime(dateTime);

	if (cmdChosenCondition == CONDITION_PROCESSDIES_ID) {
		if (processAPI != NULL || createProcess()) {
			struct sa_process procInfo;
			procInfo.pid = cmdLineArgs->getOption("program-dies").toInt();
			processAPI->getProcess(&procInfo);
			if (processAPI->error) {
				showError(processAPI->error);
			} else {
				pidProcessDies = procInfo.pid;
				gui.labelProcessDies->setText(QString::fromAscii(procInfo.filename));
			}
		}
	}
}

void MainWindow::initActionWidgets(const short cmdChosenAction) {
	int chosen;
	if (cmdChosenAction != -1)
		chosen = cmdChosenAction;
	else {
		KConfigGroup group = config->group("Actions");
		chosen = group.readEntry("LastExecuted", 0);
		if (chosen < ACTION_SHUTDOWN_ID || chosen > ACTION_KILLPROCESS_ID)
			chosen = ACTION_SHUTDOWN_ID;
	}

	actionsGroup.button(chosen)->click();

	if (!isKDE) {
		gui.checkBoxShutdownForce->setChecked(false);
		gui.checkBoxShutdownForce->setEnabled(false);
	} else if (cmdChosenAction == ACTION_SHUTDOWN_ID || cmdChosenAction == ACTION_REBOOT_ID) {
		gui.checkBoxShutdownForce->setChecked(cmdLineArgs->getOption(cmdChosenAction
				== ACTION_SHUTDOWN_ID ? "shutdown" : "reboot") == "yes");
	} else {
		KConfigGroup groupShutdown = config->group("ActionShutdown");
		gui.checkBoxShutdownForce->setChecked(groupShutdown.readEntry("Force", true));
	}

	if (!supportedSleepStates.empty()) {
		gui.radioButtonSleepStandBy->setEnabled(supportedSleepStates.contains(
				Solid::PowerManagement::StandbyState));
		gui.radioButtonSleepSuspend->setEnabled(supportedSleepStates.contains(
				Solid::PowerManagement::SuspendState));
		gui.radioButtonSleepHibernate->setEnabled(supportedSleepStates.contains(
				Solid::PowerManagement::HibernateState));

		Solid::PowerManagement::SleepState state;
		if (cmdChosenAction == ACTION_SLEEP_ID) { // cmd arg
			QString cmdSleepArg = cmdLineArgs->getOption("sleep");
			if (cmdSleepArg == "Standby")
				state = Solid::PowerManagement::StandbyState;
			else if (cmdSleepArg == "Suspend")
				state = Solid::PowerManagement::SuspendState;
			else
				state = Solid::PowerManagement::HibernateState;
		} else { // config
			KConfigGroup groupSleep = config->group("ActionSleep");
			state = (Solid::PowerManagement::SleepState) groupSleep.readEntry("Mode",
					(int) Solid::PowerManagement::HibernateState);
		}

		if (!supportedSleepStates.contains(state)) { // Just in case the mode read from the config or cmd arg is unsupported
			QSetIterator<Solid::PowerManagement::SleepState> i(supportedSleepStates);
			state = i.next();
		}

		switch (state) {
		case Solid::PowerManagement::StandbyState:
			gui.radioButtonSleepStandBy->setChecked(true);
			break;
		case Solid::PowerManagement::SuspendState:
			gui.radioButtonSleepSuspend->setChecked(true);
			break;
		default:
			gui.radioButtonSleepHibernate->setChecked(true);
		}
	}

	if (cmdChosenAction == ACTION_EXECUTECOMMAND_ID) {
		QTextStream textStream(stdin);
		gui.texteditExecuteCommand->setPlainText(textStream.readLine());
	} else if (chosen == ACTION_EXECUTECOMMAND_ID) {
		KConfigGroup groupExecuteCommand = config->group("ActionExecuteCommand");
		gui.texteditExecuteCommand->setPlainText(groupExecuteCommand.readEntry("Command"));
	} else if (cmdChosenAction == ACTION_KILLPROCESS_ID) {
		if (processAPI != NULL || createProcess()) {
			struct sa_process procInfo;
			procInfo.pid = cmdLineArgs->getOption("terminate-program").toInt();
			processAPI->getProcess(&procInfo);
			if (processAPI->error) {
				showError(processAPI->error);
			} else {
				pidKillProcess = procInfo.pid;
				gui.labelKillProcess->setText(QString::fromAscii(procInfo.filename));
			}
		}
	}
}

void MainWindow::radioButtonNetworkToggled(bool checked) {
	if (!checked)
		return;

	disconnect(gui.radioButtonNetwork, SIGNAL(toggled(bool)), this, SLOT(radioButtonNetworkToggled(bool)));

	// Populating the combobox
	QList < QNetworkInterface > interfaces = QNetworkInterface::allInterfaces();
	int i;
	for (i = 0; i < interfaces.size(); i++) {
		const QNetworkInterface interface = interfaces[i];
		if (interface.flags() & QNetworkInterface::IsUp)
			gui.comboBoxNetworkInterface->addItem(interface.name());
	}

	// Trying to find out which network interface i should choose by default
	QString interfaceName;
	KConfigGroup groupNetwork = config->group("ConditionNetwork");
	if (cmdLineArgs->isSet("interface")) // First the cmd arguments
		interfaceName = cmdLineArgs->getOption("interface");
	else if (groupNetwork.hasKey("Interface")) // Trying with the config
		interfaceName = groupNetwork.readEntry("Interface");
	else { // Choosing the interface with more traffic
		networkInterfaceAPI = new NetworkInterface();
		struct sa_net_interface* interfaces;
		uint16_t written;
		interfaces = networkInterfaceAPI->getNetworkInterfaces(&written);
		if (networkInterfaceAPI->error) {
			showError(networkInterfaceAPI->error);
		} else {
			uint64_t traffic = 0;
			for (i = 0; i < written; i++) {
				if (interfaces[i].received_bytes + interfaces[i].sent_bytes > traffic) {
					interfaceName = interfaces[i].name;
					traffic = interfaces[i].received_bytes + interfaces[i].sent_bytes;
				}
			}
		}
	}
	i = gui.comboBoxNetworkInterface->findText(interfaceName);
	gui.comboBoxNetworkInterface->setCurrentIndex(i == -1 ? 1 : i);
}

void MainWindow::radioButtonPlayAlarmToggled(bool checked) {
	if (!checked)
		return;

	disconnect(gui.radioButtonPlayAlarm, SIGNAL(toggled(bool)), this, SLOT(radioButtonPlayAlarmToggled(bool)));
	availableSounds = new QStringList(KGlobal::dirs()->findAllResources("sound", "Sentinella/*",
			KStandardDirs::NoDuplicates));
	availableSounds->sort();
	// Loading config for the combobox
	KConfigGroup groupPlayAlarm = config->group("ActionPlayAlarm");
	QString savedFileName = groupPlayAlarm.readEntry("File");
	int savedIndex = -1;
	// Populating the combobox
	for (int i = 0; i < availableSounds->size(); i++) {
		if (savedIndex == -1 && savedFileName.compare(availableSounds->at(i)) == 0)
			savedIndex = i;
		QFileInfo fileInfo = QFileInfo(availableSounds->at(i));
		gui.comboBoxPlayAlarm->addItem(fileInfo.baseName());
	}

	if (savedIndex != -1)
		gui.comboBoxPlayAlarm->setCurrentIndex(savedIndex);
}

void MainWindow::chooseProcess(bool isAction) {
	KDialog* procsDialog = new KDialog(this);
	procsDialog->setCaption(i18nc("@title:window", "Choose a program"));
	procsDialog->setButtons(KDialog::Ok | KDialog::Cancel);

	KSysGuardProcessList* processList = new KSysGuardProcessList(procsDialog);
	processList->setUpdateIntervalMSecs(1500);
	processList->setKillButtonVisible(false);
	if (isAction)
		processList->setState(ProcessFilter::OwnProcesses);
	processList->treeView()->setSelectionMode(QAbstractItemView::SingleSelection);
	procsDialog->setMainWidget(processList);

	if (procsDialog->exec() == QDialog::Accepted) {
		QList<KSysGuard::Process*> selectedList = processList->selectedProcesses();
		if (!selectedList.isEmpty()) {
			KSysGuard::Process* selected = selectedList.at(0);
			int index = selected->name.indexOf(' ');
			if (isAction) {
				pidKillProcess = selected->pid;
				gui.labelKillProcess->setText(index == -1 ? selected->name : selected->name.left(
						index));
			} else {
				pidProcessDies = selected->pid;
				gui.labelProcessDies->setText(index == -1 ? selected->name : selected->name.left(
						index));
			}
		}
	}

	delete processList;
	delete procsDialog;
}

void MainWindow::programDiesClicked() {
	chooseProcess(false);
}

void MainWindow::killProcessClicked() {
	chooseProcess(true);
}

void MainWindow::startStopClicked() {
	if (condition == NULL)
		startClicked();
	else
		stopClicked();
}

void MainWindow::startClicked() {
	// validating actions
	int actionCheckedId = actionsGroup.checkedId();
	switch (actionCheckedId) {
	case ACTION_SLEEP_ID:
		if (!gui.radioButtonSleepStandBy->isChecked() && !gui.radioButtonSleepSuspend->isChecked()
				&& !gui.radioButtonSleepHibernate->isChecked()) {
			KMessageBox::sorry(this, i18nc(
					"@info The user has to choose between hibernation, standby and Suspend modes",
					"You must select sleep mode."));
			return;
		}
		break;
	case ACTION_EXECUTECOMMAND_ID:
		if (gui.texteditExecuteCommand->toPlainText().isEmpty()) {
			KMessageBox::sorry(this, i18n("A command must be specified."));
			return;
		}
		break;
	case ACTION_KILLPROCESS_ID:
		if (pidKillProcess == 0) {
			KMessageBox::sorry(this, i18n("You must select a program to kill."));
			return;
		}
		if (processAPI != NULL || createProcess()) {
			if (!procExists(pidKillProcess))
				return;
		}
	}

	// validating conditions
	qint64 msecs = 0;
	int conditionCheckedId = conditionsGroup.checkedId();
	if (conditionCheckedId == CONDITION_CPU_ID || conditionCheckedId == CONDITION_MEMORY_ID
			|| conditionCheckedId == CONDITION_NETWORK_ID) {
		QTime time;
		switch (conditionCheckedId) {
		case CONDITION_CPU_ID:
			time = gui.timeEditCPU->time();
			break;
		case CONDITION_MEMORY_ID:
			time = gui.timeEditMemory->time();
			break;
		case CONDITION_NETWORK_ID:
			time = gui.timeEditNetwork->time();
			break;
		}

		msecs = (time.minute() * 60 + time.second()) * 1000;
		if (msecs == 0) {
			KMessageBox::sorry(this, i18nc("@info The user has chosen a wrong time: 00:00",
					"The specified time must be greater than zero."));
			return;
		}
	}

	switch (conditionCheckedId) {
	case CONDITION_CPU_ID:
		condition = new CPU(msecs, 2000, gui.spinBoxCPU->value(), gui.comboBoxCPU->currentIndex()
				== 0);
		break;
	case CONDITION_MEMORY_ID:
		condition = new Memory(msecs, 4000, gui.spinBoxMemory->value() * 1048576,
				gui.comboBoxMemory->currentIndex() == 0);
		break;
	case CONDITION_NETWORK_ID:
		if (networkInterfaceAPI == NULL)
			networkInterfaceAPI = new NetworkInterface();
		condition = new Network(networkInterfaceAPI, msecs, 2000,
				gui.comboBoxNetworkInterface->currentText(), gui.spinBoxNetwork->value() * 1024,
				gui.comboBoxNetworkHigher->currentIndex() == 0,
				gui.comboBoxNetworkDirection->currentIndex() == 0 ? DOWNLOAD : UPLOAD);
		break;
	case CONDITION_DATETIME_ID:
		msecs = QDateTime::currentDateTime().msecsTo(gui.kdatetimewidgetDateTime->dateTime());
		if (msecs <= 0) {
			KMessageBox::sorry(this, i18nc("@info",
					"You have selected a date in the past. Please input a future date."));
			return;
		}
		condition = new DateTime(msecs);
		break;
	case CONDITION_PROCESSDIES_ID:
		if (pidProcessDies == 0) {
			KMessageBox::sorry(this, i18nc("@info", "You must select a program to monitor."));
			return;
		}
		if (processAPI != NULL || createProcess()) {
			if (!procExists(pidProcessDies))
				return;
		}
		condition = new ProcessDies(processAPI, 2000, 2000, pidProcessDies);
	}

	connect(condition, SIGNAL(conditionAccomplished()), this, SLOT(conditionAccomplished()));
	connect(condition, SIGNAL(conditionFailed(int)), this, SLOT(conditionFailed(int)));
	gui.groupBoxConditions->setEnabled(false);
	gui.groupBoxActions->setEnabled(false);
	gui.pushButtonStartStop->setText(i18nc("@action:button Stop the running task", "Stop"));
	gui.pushButtonStartStop->setIcon(
			QIcon(iconLoader->loadIcon("process-stop", KIconLoader::Small)));
	saveConfig(conditionCheckedId, actionCheckedId, msecs);
}

inline void MainWindow::saveConfig(int conditionCheckedId, int actionCheckedId, qint64 msecs) {
	KConfigGroup group = config->group("Conditions");
	group.writeEntry("LastExecuted", conditionCheckedId);
	KConfigGroup configGroup;
	switch (conditionCheckedId) {
	case CONDITION_CPU_ID:
		configGroup = config->group("ConditionCPU");
		configGroup.writeEntry("Higher", gui.comboBoxCPU->currentIndex());
		configGroup.writeEntry("Percentage", gui.spinBoxCPU->value());
		configGroup.writeEntry("Time", gui.timeEditCPU->time().toString("mm:ss"));
		break;
	case CONDITION_MEMORY_ID:
		configGroup = config->group("ConditionMemory");
		configGroup.writeEntry("Higher", gui.comboBoxMemory->currentIndex());
		configGroup.writeEntry("Amount", gui.spinBoxMemory->value());
		configGroup.writeEntry("Time", gui.timeEditMemory->time().toString("mm:ss"));
		break;
	case CONDITION_NETWORK_ID:
		configGroup = config->group("ConditionNetwork");
		configGroup.writeEntry("Direction", gui.comboBoxNetworkDirection->currentIndex());
		configGroup.writeEntry("Higher", gui.comboBoxNetworkHigher->currentIndex());
		configGroup.writeEntry("Amount", gui.spinBoxNetwork->value());
		configGroup.writeEntry("Interface", gui.comboBoxNetworkInterface->currentText());
		configGroup.writeEntry("Time", gui.timeEditNetwork->time().toString("mm:ss"));
		break;
	case CONDITION_DATETIME_ID:
		configGroup = config->group("ConditionDateTime");
		configGroup.writeEntry("Interval", msecs / 1000);
		break;
	}

	group = config->group("Actions");
	group.writeEntry("LastExecuted", actionCheckedId);
	switch (actionCheckedId) {
	case ACTION_SHUTDOWN_ID:
	case ACTION_REBOOT_ID:
		configGroup = config->group("ActionShutdown");
		configGroup.writeEntry("Force", gui.checkBoxShutdownForce->isChecked());
		break;
	case ACTION_SLEEP_ID:
		Solid::PowerManagement::SleepState state;
		configGroup = config->group("ActionSleep");
		if (gui.radioButtonSleepStandBy->isChecked())
			state = Solid::PowerManagement::StandbyState;
		else if (gui.radioButtonSleepSuspend->isChecked())
			state = Solid::PowerManagement::SuspendState;
		else
			state = Solid::PowerManagement::HibernateState;
		configGroup.writeEntry("Mode", (int) state);
		break;
	case ACTION_EXECUTECOMMAND_ID:
		configGroup = config->group("ActionExecuteCommand");
		configGroup.writeEntry("Command", gui.texteditExecuteCommand->toPlainText());
		break;
	case ACTION_PLAYALARM_ID:
		configGroup = config->group("ActionPlayAlarm");
		configGroup.writeEntry("File", availableSounds->at(gui.comboBoxPlayAlarm->currentIndex()));
	}
}

void MainWindow::stopClicked() {
	deleteConditionLater();

	gui.groupBoxConditions->setEnabled(true);
	gui.groupBoxActions->setEnabled(true);
	gui.pushButtonStartStop->setText(i18nc("@action:button Verb in imperative form", "Start"));
	gui.pushButtonStartStop->setIcon(QIcon(iconLoader->loadIcon("arrow-right", KIconLoader::Small)));
}

void MainWindow::conditionAccomplished() {
#define PASSIVE_POPUP_TITLE i18nc("@info The name of the app in the title of a passive popup", "Sentinella")
	deleteConditionLater();

	int actionCheckedId = actionsGroup.checkedId();
	switch (actionCheckedId) {
	case ACTION_SHUTDOWN_ID:
	case ACTION_REBOOT_ID:
		action = new Shutdown(isKDE, gui.radioButtonShutdown->isChecked(),
				gui.checkBoxShutdownForce->isChecked());
		if (gui.radioButtonShutdown->isChecked())
			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
					"Condition met. Halting the system."), this);
		else
			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
					"Condition met. Rebooting the system."), this);
		break;
	case ACTION_SLEEP_ID:
		Solid::PowerManagement::SleepState state;
		if (gui.radioButtonSleepStandBy->isChecked()) {
			state = Solid::PowerManagement::StandbyState;
			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
					"Condition met. Setting the system's state to standby."), this);
		} else if (gui.radioButtonSleepSuspend->isChecked()) {
			state = Solid::PowerManagement::SuspendState;
			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
					"Condition met. Suspending the system."), this);
		} else {
			state = Solid::PowerManagement::HibernateState;
			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
					"Condition met. Hibernating the system."), this);
		}
		action = new Sleep(isKDE, state);
		break;
	case ACTION_EXECUTECOMMAND_ID:
		action = new ExecuteCommand(gui.texteditExecuteCommand->toPlainText());
		KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
				"Condition met. Executing command."), this);
		break;
	case ACTION_PLAYALARM_ID:
		action = new PlayAlarm(availableSounds->at(gui.comboBoxPlayAlarm->currentIndex()));
		KPassivePopup::message(PASSIVE_POPUP_TITLE,
				i18nc("@info", "Condition met. Throwing alarm."), this);
		break;
	case ACTION_KILLPROCESS_ID:
		action = new KillProcess(pidKillProcess);
		KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
				"Condition met. Killing the selected program."), this);
		break;
	}
	action->execute();

	if (actionCheckedId == ACTION_PLAYALARM_ID) {
		KMessageBox::information(this, i18n("Close this message to turn off the alarm."));
		((PlayAlarm*) action)->stop();
		delete action;
		action = NULL;
	}

	gui.groupBoxConditions->setEnabled(true);
	gui.groupBoxActions->setEnabled(true);
	gui.pushButtonStartStop->setText(i18nc("@action:button Verb in imperative form", "Start"));
	gui.pushButtonStartStop->setIcon(QIcon(iconLoader->loadIcon("arrow-right", KIconLoader::Small)));
}

void MainWindow::conditionFailed(int error) {
	deleteConditionLater();
	showError(error);
}

void MainWindow::deleteConditionLater() {
	condition->stop();
	condition->deleteLater();
	disconnect(condition, SIGNAL(conditionAccomplished()), this, SLOT(conditionAccomplished()));
	disconnect(condition, SIGNAL(conditionFailed(int)), this, SLOT(conditionFailed(int)));
	condition = NULL;
}

void MainWindow::showError(const int error) {
	switch (error) {
	case ENOSYS:
		KMessageBox::error(this, i18n("Feature not available."));
		break;
	case ENODEV:
		KMessageBox::error(this, i18n("No such device."));
		break;
	case ENOMEM:
		KMessageBox::error(this, i18n("Not enough memory."));
		break;
	case ESRCH:
		KMessageBox::error(this, i18n("No such process."));
		break;
	case EIO:
		KMessageBox::error(this, i18n("I/O error."));
		break;
	default:
		KMessageBox::error(this, i18n("An internal error occurred."));
	}
}

void MainWindow::showAboutApp() {
	KAboutApplicationDialog* aboutAppDialog = new KAboutApplicationDialog(&aboutData, this);
	aboutAppDialog->exec();
	delete aboutAppDialog;
}

void MainWindow::closeEvent(QCloseEvent *event) {
	if (!event->spontaneous())
		return;
	setVisible(false);
	event->ignore();
}

bool MainWindow::createProcess() {
	processAPI = new Process();
	if (processAPI->error != 0) {
		showError(processAPI->error);
		delete processAPI;
		processAPI = NULL;
		return false;
	}
	return true;
}

bool MainWindow::procExists(pid_t pid) {
	if (processAPI->processExists(pid))
		return true;

	showError(processAPI->error);
	return false;
}

void MainWindow::actionButtonClicked(int id) {
	gui.stackedWidgetActions->setCurrentIndex(id == 0 ? 0 : id - 1);
}

#include "MainWindow.moc"
